#encoding 'UTF-8'
=begin
/***************************************************************************
 *   ©2011-2016 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************
=end

=begin
	An object of this class indicates activity. It does so asynchronously
	until interrupted.

	Examples:
	----------------------------------------
	
	# 1) start default-animation immediately

	width = 20
	2.times do
		puts "I am busy, pse wait..."
		thr = BusyIndicator.new(true, width)
		sleep 3
		thr.stop("Okay")
	end
	
	# 2) create object, start animation later

	bi = BusyIndicator.new(false, width)
	bi.run
	sleep 3
	bi.stop("Stopped")

	# 3) change the width of the animation
	
	bi.width = 8
	bi.run
	sleep 3
	bi.stop("stopped again")

	# 4) use a diffferent animation sequence

	bi = BusyIndicator.new(true, 20, animation=>%w~... ,,, ;;; ::: ||| 777 >>> ___~)
	sleep 3
	bi.stop

	# 5) animate the output of a block

	tr = %w"a b c d e f g"
	i = nil
	bi = BusyIndicator.new(true, 20 ) do 
		i = (i && i < tr.size ? i + 1 : 0)  	
		tr[i] 
	end
	sleep 3
	bi.stop
=end
class BusyIndicator
	attr_writer :width

	# Constructor. All arguments are optional; set 'start' to true to start
	# immediately, define the width of the output as a number of visible
	# characters. The variable 'animation' can contain an array of symbols which 
	# will be printed sequentially to create an animation.
	# 
	# If a block is given, its output will be printed repeatedly, instead
	# of the animation-symbols. In the block, consider protecting shared
	# ressources with a Mutex.
	
	def initialize(start = true, width = nil, animation = nil, &b)
		
		# @animation = %w"OOO ooo ___ ooo"
		# @animation = %w"||||||| ||||||( |||||(- |||(-)| ||(-)|| |(-)||| (-)|||| -)||||| )|||||"

		@animation = (animation || %w"||| ||| ((( <<< ––– ––– >>> )))")
		@proc = b if b

		l = @animation.max {|a, b| a.length <=> b.length}.length

		@width = (width && width >= l ? width : l)
		@thr = busy_indicator(@width) if start
	end
	# start the animation	
	def run() 
		@thr = busy_indicator(@width)
	end
	# stop the animation and print the comment.
	def stop(comment = nil)
		@thr.terminate
		@thr.join
		print ("\b" * @width)
		print ("%+#{@width}s\n" %comment) if comment
	end
	private

	def animate(width)
		# nw = width	
		loop do 
			@animation.each do |s|
				print ("\b" * (width))
				print "%+#{width}s" %s
				sleep 0.1
			end
		end
	end

	def evolve
		loop do
			begin
			sv = @proc.call
			if(@width < sv.length)
				@width += sv.length/2
			end
			print "\b" * @width
			print "%+#{@width}s" %sv
			rescue Exception
				stop('thread ended abnormally')
			end
			# sleep 0.5
		end
	end

	# called by the method run().
	def busy_indicator( width )
		tobj = Thread.new()  do  
			if(@proc)
				evolve	
			else
				animate width
			end
		end
	end
end
# ============TEST===========
if($0 == __FILE__)
	
	# 1) basic use with a given width and a default-animation
	width = 20
	2.times do
		puts "I am busy, pse wait..."
		thr = BusyIndicator.new(true, width)
		sleep 3
		thr.stop("Okay")
	end

	# 2) do not start immediately
	bi = BusyIndicator.new(false, width)
	# ... but now.
	bi.run
	sleep 3
	bi.stop("Stopped")

	# 3) variate width
	bi.width = 8
	bi.run
	sleep 3
	bi.stop("stopped again")

	# 4) define a different animation sequence
	bi = BusyIndicator.new(true, 20, %w~... ,,, ;;; ::: ||| 777 >>> ___~)
	sleep 3
	bi.stop('A joke:')
	
	# 5) animate the output from a block.
	tr = %w"A dog walks into the saloon: I'm looking for the man who shot my paw..."
	i = nil
	bi = BusyIndicator.new(true, 20 ) do 
		i = (i && i < tr.size ? i + 1 : 0)  	
		sleep 0.5
		tr[i] 
	end
	sleep 9
	bi.stop
	#6 ) Show a timeout countdown, irrespective of the BusyIndicator's
	#    preset sleep-time
	timeout = 5
	t = Time.now.sec
	bi = BusyIndicator.new(true) do 
		sleep 0.5
		timeout -= 1 if t != Time.now.sec
		t = Time.now.sec
		timeout.to_s
	end
	sleep timeout 
	bi.stop("... ah. Oh. Okay, got it.")

end

#EOF
