#encoding: UTF-8 
=begin
/***************************************************************************
 *   ©2011-2016 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end
require 'logger'
require_relative 'file_checking'

=begin Creates a member @log and precede its output with the name of the class
of the object. 
Example for a class-level logger:
# --------------------
class TClass
	self.extend(Logging)
	@@log = init_logger(STDOUT)
	def test_log
		@@log.info('class-level logger called from instance: ' << @@log.to_s)
		@log = @@log
		@log.info('AGAIN: class-level logger called from instance: ' << @log.to_s)
	end
	def self::test_log
		@log.info('class-level logger called from class: ' << @log.to_s)
		@@log.info('AGAIN: class-level logger called from class: ' << @@log.to_s)
	end
end
#---------------------
Example for a object-level logger:
ATTN! This means 1 logger per object.
# --------------------
class TClass
	include Logging
	def initialize
		init_logger(STDOUT, Logger::DEBUG)
	end
	def test_log
		@log.debug('called test_log() ')
	end
end
=end
module Logging
	include File_Checking

	@@have_log = false
	@@LOG_CONF = File.dirname(File.absolute_path(__FILE__)) << File::Separator << 'log.conf'

	# Call this method in an instance-method (e.g. initialize() ) to define the
	# object-level logger; i.e. an object-specific member @log.
	# Call this method within the class-definition for a class-level logger; i.e.
	# a member @log for class-level acces.
	# The method returns the logger, so you can actually do what you want with it.
	def init_logger(target = STDOUT, level = Logger::INFO)
		# Prepare for a class-level logger. This is actually quite cool.
		
		# ---> Ingeniuous code starts here
		cn = (self.class == Class ? name : self.class.name)
		# <--- Ingeniuous code ends here
		
		# allow to override the set log-levels with an
		# external configuration (log.conf).
		log_conf(cn)
		# Or use the defaults as set here or elsewhere...

		@level ||= level 
		@target ||= target

		@log = Logger.new(@target)
		@log.level = @level

		@log.formatter = proc do |severity, datetime, progname, msg|
			t = Time.now
			"#{cn}: #{severity} #{t.hour}-#{t.min}-#{t.sec}: #{msg}\n"
		end
		if ! @@have_log
			@log.debug cn.dup << ' reading logging-configuration from ' << @@LOG_CONF	
			@@have_log = true
			@log.debug('level is ' << level.to_s)
		end
		return @log
	end

	# Set the log-target to an IO object.
	def log_target=(target)
		@target = target
		@log = Logger.new(@@target)
		@log.level = @level
	end

	# set the log-level
	def log_level=(level)
		@level = level
		@log.level = @level
	end

	private

	# Override or set the log-level and target-device, as set in a file 'log.conf'.
	# I do not like the look of this, but it works just the way I want it to.
	# "HEAVANS! Isn't there a standard way to do this in Ruby, for Christ's sake?", you say.
	# Heck, I don't care. <= Read that again, I say.
	def log_conf(cn = nil)
		config = level = target = nil
		# puts 'log-config is in ' << @@LOG_CONF
		if(File::exist?(@@LOG_CONF) )
			begin
				conf = File.read(@@LOG_CONF)
				config = instance_eval(conf)
			rescue Exception => ex
				STDERR.puts "WARNING! Cannot evaluate the logger-configuration!" << ' ' << ex.message
				STDERR.puts "Default log-levels apply."
			end
		else
			puts "Default log-levels apply."
		end

		if(config && config.respond_to?(:to_hash) )
			config.default = nil
			if cn
				config = config[cn.to_sym]
			else
				config = config[self.class.name.to_sym]
			end

			if(config )
				if(config.respond_to?(:to_ary) && config.size == 2)
					@level, @target = config
					@target.downcase!
					logdir = File.dirname(@target)
					msg = file_check(logdir, :exist?, :directory?, :writable?)
					if(msg) 
						STDERR.puts "WARNING! A logfile for '%s' cannot be written to %s (%s)!" %[self.class.name, logdir, msg] 
						@target = nil
					end
				else
					@level = config
				end	
			end
		end
	end
end

######### test
if __FILE__ == $0
	class TClass
		# class level ---->
		self.extend(Logging)
		@@log = init_logger(STDOUT, Logger::INFO)
		# <------
		# object-level ---->
		include Logging
		# <---------

		def test_log
			@@log.info('class-level logger called from instance: ' << @@log.to_s)
			#@log = @@log # works too
			@log = TClass.class_eval{@log}
			@log.info('AGAIN: class-level logger called from instance: ' << @log.to_s)
			@log.debug("you won't see this on log-level INFO")

			# object-level ---->
			init_logger
			# <-----------
			@log.info("That's a different thing: " << @log.to_s << " - object-level logger!")

		end
		def self::test_log
			@log.info('class-level logger called from class: ' << @log.to_s)
			@@log.info('AGAIN: class-level logger called from class: ' << @log.to_s)
		end
	end

	TClass.new.test_log # class-logger + 1st object-logger
	TClass.new.test_log # same class-logger + 2nd object-logger

	TClass::test_log # same class-logger
	puts 'And just say it once clearly: THIS IS COOOL!!'
end
#EOF
